# -*- coding: UTF-8 -*-
# Copyright©2022 xiangyuejia@qq.com All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
"""
from typing import Dict, Union, List, Any, NoReturn, Tuple
import re
from aitool import pip_install, is_all_chinese, singleton, exe_time


@singleton
class POSFilter:
    def __init__(self, method='jieba'):
        self.method = method
        if self.method == 'jieba':
            import jieba.posseg as psg
            self.pos = psg.cut
        else:
            import jieba.posseg as psg
            self.pos = psg.cut

    def get_pos(self, text):
        if self.method == 'jieba':
            # jieba中词性的定义：https://gist.github.com/hscspring/c985355e0814f01437eaf8fd55fd7998
            word_pos = list(self.pos(text))
            word_pos = [(_.word, _.flag) for _ in word_pos]
            return word_pos

    def is_pos_valid(self, text, show_detail=False, target=None, vv=None, ths_target=0.63, ths_vv=0.71):
        if target is None:
            # jieba中词性的定义：https://gist.github.com/hscspring/c985355e0814f01437eaf8fd55fd7998
            target = ['NR', 'NT', 'NN', 'VV', 'n', 'ng', 'nr', 'nrfg', 'nrt', 'ns', 'nt', 'nz',
                      'r', 'rg', 'rr', 'rz', 'v', 'vd', 'vg', 'vi', 'vn', 'vq']
        if vv is None:
            vv = ['VV', 'v', 'vd', 'vg', 'vi', 'vn', 'vq']
        if show_detail:
            print('target', target)
            print('vv', vv)
            print('ths_target', ths_target)
            print('ths_vv', ths_vv)

        word_pos = self.get_pos(text)

        pos_count = len(word_pos)
        if pos_count == 0:
            return False

        target_count = 0
        vv_count = 0
        for p in word_pos:
            if p in target:
                target_count += 1
            if p in vv:
                vv_count += 1

        target_score = target_count / pos_count
        vv_score = vv_count / pos_count
        print(text, target_score, vv_score)
        if target_score >= ths_target and vv_score <= ths_vv:
            if show_detail:
                print(text, 'True', target_score, vv_score, word_pos)
            return True
        else:
            if show_detail:
                print(text, 'False', target_score, vv_score, word_pos)
            return False


if __name__ == '__main__':
    data = [
        '你们这是什么鬼为什么QQ看不到的东西你们能看到？？？',
        '手机相册是空的，照片早已删除，但是却保存在抖音相册里，而且无法删除，本人已经尝试过注销账号，注销期限过后，重新申请账号，照片还在抖音相册里，因为这两张照片，是本人身份证信息，非常重要。为避免信息泄露造成个人财产损失，请平台尽快帮我解决问题。',
        '我手机格式化过以前的照片都没了。抖音的那年今日还能看到。为什么往年今日还能获取到。请问抖音是从哪里获取的呢。',
        '有关证件我在手机相册里早删了。可在抖音相册里还有。你设计一下抖音里的相册同手机里的相册的照片视频一样很容易的删除才可以',
        '系统相册照片数量和抖音app上显示的数量不一致',
        '我清理的是你们抖音APP的空间，为何返回后查看到我手机相册中的图片也跟随消失了，你们平台还可以访问我的手机空间，且自作主张把我数据删除？真的是无法无天了啊',
        '我跟我朋友的私信消息记录为何突然消失了呢，而且只消失了这一个，我没有操作删除，是你们平台误删了吧，你们不仅可以查看到我的私信记录，查看以后还给我删除，希望给一个合理的解释。',
        '在登录账号的时候，突然出现了人脸验证弹窗提示，但是我的摄像头权限设置是关闭的，为何点击验证还是可以识别到我的脸呢？',
        '我的摄像头权限一直是关闭的，为何作品中还是会出现自拍的作品呢？而且刚刚操作也是可以进行自拍录制作品的？',
        '我明明已经关闭了你们APP的摄像头识别权限，为什么还可以跟别人进行视频聊天，中途也没有开启提醒弹窗提示呀？',
        'PC端搜索 用户 关键词时 搜索出 自己的 实名了？这是 什么问题 ，关键词是  胖子冷知识',
        '自己通讯录好友 出现自己的名字 但是不是自己的账号 朋友侧也是显示同一个账号 影响自己生活了',
        '显示是实名而不是昵称。才知道我发的视频有我的名字。没有昵称。这显示我的昵称。只是别人的手机上就显示的是名字。',
        '我们自己的电话号码，被人上传抖音说是车管所电话，每天接到电话咨询的，已对我们的生活照成影响。望贵平台尽快下架处理。',
        '抖音在使用时过于频繁地读取位置信息，读取次数比其他软件多很多。',
        '为什么抖音要一直获取我的位置信息',
        '我登录抖音的时候，突然发现我的账号头像也没了，资料也没了，抖音名字也没了，然后我给朋友说话也不能说了，私信什么功能也被关闭了，然其中一部分我个人曾经隐藏过的信息全部都被公开了，凭什么把我的自己的隐私给我公开了？',
        '我从来没有在你们平台进行实名认证，为何刚刚点击去认证后竟然会出现我的身份证号呢，你们平台是通过哪里自动识别了我的身份证信息呢？太可怕了吧~',
        '我一直都是私密账号的状态，为何还收到了陌生好友的生日祝福呢，你们在哪里暴露了我的生日和信息呢',
        '你们平台竟然有个功能，是系统弹窗提示他人生日，让我去送祝福，这岂不是暴露他人的隐私吗，经过我们允许了吗？这个是按照个人信息自动推送的吗？',
        '我最近一直在参加你们平台的活动，累计一些收益终于可以提现了，但是我点击进行提现时，自动出现银行卡提现功能而且上边绑定了我的银行卡号，我从未在你们平台绑定过的，这是怎么回事呢？',
        '最近在你们平台使用我的银行卡下单后，就频繁收到一些让我办理贷款和信用卡的骚扰电话，你们平台对用户的隐私做不到保障啊，等收到货后我只能卸载了',
        '我账号可能被盗了。一直在其他地方登陆。并且把其他地方登陆退出。他还是一直验证码能登陆。',
        '我今天拍了几张照片。怎么就自动上传到抖音了。我就拍了几张照片。它就变成视頻自动上传了。但我害怕。我害一些隐私照片自动上传就闹大了。',
        '很震惊，自动发表了视频，让我脸都丢尽了，给我个合理的答复，不然我会上诉。',
        '为什么会识别手机相册里最新的照片自动发日常？明明本人没有发，自动读取功能就算了，为什么没经过本人同意自动发日常视频？',
        '你好相册照片为什么为什么自动发布了',
        '自己今天发布一个作品，当时自己选完照片之后还看了一遍是没有其他照片的，当时下午自己在看的时候发现抖音把自己手机内的机密照片进行了连带发布，自己觉得非常不合理',
        '我的账号在我不知情的情况下，无缘无故自动读取我手机里的照片，把我的隐私照片发布成了作品，曝光在了抖音上，这是什么原因！',
        '有的时候我看别人抖音，我截个图，不知不觉怎么上传到我的主页里面，这个作品连我，自己都不知什么时候上传的',
        '今天凌晨我根本就没有在抖音上发作品。 睡了一觉起来发现账号违规了。显示我发了几条违规视频，封禁了，真的是莫名其妙啊！！！！！我一直在浙江宁波，账号ID怎么可能会在湖北？？？？请麻烦看到我的反馈处理一下我的封禁问题好吗？',
        '我现在不想让再自动给我发送团购视频了。之前公司是给我们都开通了团购达人。但是关于团购的视屏都不是我发的。',
        '私信不知道 为什么自己发出来红包花了5千多 我没有发啊，有账单截图记录',
        '现在两个账号登录是重复的。但是一发作品就是另一个账号。账号能正常登录但是一发布作品。作品就发布在账号一去了。',
        '抖音创作者中心最近是改版了么。抖音会自动更新微信公众号的内容到抖音么。不知道怎么会出现这个视频。我们发布的是这样的动态视频。',
        'APP端自动发布的问题',
        'PC端自动发布的问题：1.我抖音为啥会自己发作品？ 2.下午的时候我连抖音都没有用过，我的账号自己在发作品.还是黄色群聊作品？ 这是什么问题',
        '小程序自动发布的问题',
        '"这个链接不知道为什么被自动复制了，看评论区很多人都有同样的体验，不知道是不是啥黑产策略',
        '3.38 Kjc:/ 复制打开抖音，看看【懂视的作品】ts是什么梗 # 热梗 # ts"',
        '飞书和微信点击抖音链接跳转后，在抖音都出了剪切板的弹窗',
        '看着视频也蹦出来链接。一进抖音就出来链接。这是上午打开抖音弹出来的链接。刷着视频也蹦出来。有一段时间没出现。现在又开始了。',
        '有病吗抖音。真的有气了。我真的想砸手机了。我没打开抖音。它自己莫名其妙的弹出来了。你说这抖音是不是有病。',
        '我问一下那个抖抖音极速版，我这手机老也老也跳老也跳出来，我没打开它自动就打开，咋回事啊？',
        '老是自动打开抖音，关都关不掉，一会就自己开了，一会就自己开了',
        '抖音极速版看完广告会自动跳转到其他app。看了个美团的推广广告。然后广告结束自动跳转打开了美团app。其他的不会。',
        '抖音落地页没有手动点击，自动跳转微信',
        '广告视频的评论区，评论中有超链接，第一次点击后直接跳转到微信列表中，无弹窗提示，第二次进入后直接弹出来加微信的二维码，站外引流',
        '为什么会自己加上去。我自己没有加上去的。但想知道是谁加上去的。好的麻烦了。我自己绝对没有加上去的。因为我从来就没有用过这类话题。',
        '为什么新注册的号，在消息这边还是会有大号互关的好友呀',
        '手机打开抖音，发现抖音消息栏页面里的好友，被抖音平台自动发骚扰视频，不是本人操作，是抖音平台间隔一段时间自动发给好友视频。',
        '未分享视频，好友却收到了视频分享链接且无法观看，分享的视频我未观看过。',
        '"我也没点过赞的视频为什么会出现在我的喜欢里边 第二张图片而且是连赞 能不能不这么搞 没赞为啥硬塞啊',
        '扫过一个二维码后点赞列表就一直出现我没有点赞过的视频"',
        '为什么评论区会出现不是我自己评论的消息呢，是他人艾特回复我的时候我才看到的~我以为账号被盗了，但是我的设备登录信息中也没有其他的设备登录记录，你们平台需要给我一个解释了',
        '为何我的评论区会出现系统自动回复的话，而且是一样的，你们平台这个功能从哪里关闭呢，不经过允许，你们侵犯了我权利，小心我起诉你们公司。',
        '抖音自拍设为私密，，重新关机然后开机仍然公开原因',
        '我实名制绑定的抖音账号，在很长一段时间我设置成了私密账号。可是2023年03月12日晚上我上传了40多个私密日常视频。却被我抖音粉丝列表里的陌生男人看到了，他浏览了我新发布的所有视频。我当然一气之下移除了所有粉丝。每个隐私视频下都有该男子的浏览记录，但是我忘记截图存证。因为移除粉丝就只显示该视频被点击过一次。我真的很生气，明明是私密账号发布了视频却被人看到，我认为抖音侵犯我的隐私。希望他们给我个合理的解释。为什么私密账号发布视频还要曝光我的隐私。',
        '为什么私密视频。为什么私密视频。别人搜索账号还能看到私密作品。为什么要成为私密账号。作为一个连账号都没有的用户',
        '好友那个顺序为什么一直会变。不是刷新了再看也算浏览量的吗。刷新了再看也算浏览记录啊。那为什么我仅一人可见还增加浏览量。',
        '我的视频隐藏了，怎么会自己又出来了',
        '我的账号设置了不允许陌生人找到我。可为什么企业账号还可以找到我。他还是可以搜到我的主页呢。这个是企业账号。',
        '为什么私密视频。为什么私密视频。别人搜索账号还能看到私密作品。为什么要成为私密账号。作为一个连账号都没有的用户',
        '视频设置了仅互关好友可见，但陌生人还是能看到',
        '这个视频我单独屏蔽的人为什么还能看我的作品',
        '为什么设置了不然某人看她仍然能够看到。并且能够评论。对呀她都评论了。我记得我刚刚发布就设置了她不可见。结果她还是能看到。',
        '今天我这个视频。你们抖音时不时出现了bug。请回答我的问题。屏蔽的人还是浏览了我的作品',
        '你们都有设置可以谁看不让谁看，那为什么我设置了却还有人能看见请问你们这个设置有什么问题',
        '就是我把别人屏蔽了。我已经设置了不让看。并且设置了不推荐可能认识的人。但对方还是能收到我发作品的推送。麻烦尽快啊。',
        '我一直设置的是私密账号，但是从昨天开始莫名其妙地有各种我从来没有主动点关注的商家回关我，而且我的设置被修改成公开账号，我一直设置需要我通过同意才能成为我的粉丝，请问这是怎么回事',
        '设置自己转发，为什么别人还可以转发',
        '老是被@广告，游戏黄赌毒啥的，开启了私密账号都没用，个人主页出现了被@那一栏又关不掉，要求处理去',
        '我发了几张图片。不想被人保存。权限—高级设置已经选择了不允许下载。但是可以保存图片。不是图片。',
        '我在隐私设置里面都设置了不让任何人看。收藏夹直接没有设置的界面。一个普通版本的用户可以看到我的收藏。还是别人能看到我收藏的界面',
        '后台一直能看到。这个账号注销了。可后台一直能搜到。刚才北京工作人员说了 账号已经被注销。',
        '后台跟播页面只有这个主播看不见直播数据，主播是这个经纪人旗下的，也看不到',
        '"https://voc.bytedance.net/v2/BusinContent/VocAnalysis?analysisViewCfgId=1632786',
        '啊，美女，就是有个问题，从昨天开始我的手机不是以前就直播的话，以前开直播的话，我就是只只设置部分人可以看，然后不是可以选择的嘛，对吗？',
        '嗯，对，然后我今天从昨天试到现在，我设置了部分人可以观看，但是我进去以后我的直播间还是公开的。',
        '噢对对对对对，设置好以后，我进去以后还是公开的，就所有人都可以进来。',
        '还是还是公开的，就咱们来电手机绑定的这个抖音号吗？',
        '嗯，对，这个抖音号还有另外一个也是这样，我两个号都试了。"',
        '"为什么我的抖音仅一个人可见 其他人也能看到，我发现后就及时把他私密了，而且看的那个人只是普通抖音好友，我和他不认识，不存在之前他留在我仅谁谁可见的列表里。',
        '我本来发了私密仅自己可见的，后来就只转发了我这一个朋友看，然后出现了第二个人看我作品，为什么会出现第二个人看我作品"',
        '就是我把自己的账号设置了私密账号。突然发现别人搜索我的账号居然可以看见我的视频。不应该是可以搜索到。恰好是陌生用户可以看见视频。',
        '今天我这个视频。你们抖音时不时出现了bug。请回答我的问题。屏蔽的人还是浏览了我的作品',
        '我发得做品隐私上传得，后来我直接删掉了，为什么别人还能看到，给我生活带来不便，能帮我解决吗，就这个号',
        '我这个收藏是锁的',
        '为什么删除的作品别人又能刷的到呢。我删除的作品别人两小时之后刷到的。但是别人是在10点11刷到的。好友刷到的。麻烦你查一下。',
        '明明删除了，可每次打开抖音都会出现这条作品',
        '你好，我不小心误触上传了轻颜相机的视频，非常尴尬，上传五分钟的时候就删掉了，但是现在别人还是能刷到，请问是出bug了吗',
        '我发得做品隐私上传得，后来我直接删掉了，为什么别人还能看到，给我生活带来不便，能帮我解决吗，就这个号',
        '为什么撤回的信息还能看到！！！',
        '粉丝群被人发营销红包添加微信。怎么设置下不能发红包和点赞啊。我都设置群禁言了。',
        '"我在手机权限里并未开通 定位服务及语音但抖音依然能够窥探个人隐私 比如在烟台机场抖音搜索同城推送的是烟台 未经过允许',
        '第二我和朋友聊天谈及离婚内容 抖音推送离婚内容视频"',
        '抖音未经允许给我录音，手机提示应用敏感行为，使用场景完完全全用不到录音，我那时候刚抖音支付实名认证然后的话想注销，然后又点回来刷视频，根本用不到录音功能，连你们抖音客服都说不会出现这种情况的，希望抖音给出合理解释，不然就向人民法院提起诉讼',
        '为什么我的帐号会有其他人的作品？',
        '最近抖音老是给我推送这种莫名其妙的东西。我也没有和这些聊过啊。莫名其妙的通知。根本没有跟这些联系过。然后点开又什么都没有。',
        '自己艾特了好友但是会有陌生人收到',
        '不是我的好友。也不是我的账号。聊天内容会通知给我。这是什么原因。',
        '登陆了别人的手机，然后登回自己手机后，通讯录推荐都是别人的，再登回自己手机就这样了。打开关闭无数次了。',
        '我遇到了诡异事件，在我消息列表中出现了多个陌生人的私信消息，而且跟我沟通都是牛头不对马嘴的，感觉我断片了的样子',
        '我的消息列表中怎么突然多了好几个陌生人的消息，而且是很早以前，不是最新的，好奇怪。',
        '为何收藏列表中出现了很都我没见过的作品，而且数量突然暴涨，你们平台出现BUG了吗',
        '我今天的历史记录中都是我看过的作品，但是收藏中的作品我却没见过也没在浏览记录中展示，我的账号被盗了也不符合逻辑呀',
    ]
    x = POSFilter()
    for d in data:
        print(d)
        print(x.get_pos(d))

